const std = @import("std");

const MKIT_BUILTIN = struct {
    __MKIT_PROJ_TAGLINE__: []const u8 = "",
    __MKIT_PROJ_GIT_LASTHASH__: []const u8 = "",
    __MKIT_PROJ_VERSION__: []const u8 = "",
    __MKIT_PROJ_VCS_BROWSER__: []const u8 = "",
    __MKIT_PROJ_NAME__: []const u8 = "",
    __MKIT_PROJ_GIT_LASTSUMMARY__: []const u8 = "",
    __MKIT_PROJ_LICENSE__: []const u8 = "",
    __MKIT_PROJ_PKGNAME__: []const u8 = "",
    __MKIT_PROJ_MAINTAINER__: []const u8 = "",
    __MKIT_PROJ_CODENAME__: []const u8 = "",
    __MKIT_MKIT_VERSION__: []const u8 = "",
};

fn _load_json(allocator: std.mem.Allocator) !MKIT_BUILTIN {
    const path = ".mkit/data/MKIT_BUILTIN.json";
    const fh = try std.fs.cwd().openFile(path, .{});
    const data = try fh.readToEndAlloc(allocator, 1024);
    const parsed = try std.json.parseFromSlice(MKIT_BUILTIN, allocator, data, .{
        .ignore_unknown_fields = true,
        .duplicate_field_behavior = .use_last,
    });
    return parsed.value;
}

fn maybe_load_json() ?MKIT_BUILTIN {
    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    const data = _load_json(gpa.allocator()) catch return null;
    return data;
}

/// Add MKit meta-data to import table of module *m* of build *b*
///
/// Use this in build.zig to allow your module do:
///
///     const mkit_meta = @import("mkit_meta");
///     std.log.info("proj_version={s}", .{mkit_meta.proj_version});
///
/// For the meta-data to be available, you need to make sure MKit's
/// `build` target was called first, eg.:
///
///     make build
///     zig build
///
pub fn inject_mkit_meta(b: *std.Build, m: *std.Build.Module) void {
    const data = maybe_load_json() orelse MKIT_BUILTIN{};
    const o = b.addOptions();
    o.addOption([]const u8, "mkit_version", data.__MKIT_MKIT_VERSION__);
    o.addOption([]const u8, "proj_codename", data.__MKIT_PROJ_CODENAME__);
    o.addOption([]const u8, "proj_git_lasthash", data.__MKIT_PROJ_GIT_LASTHASH__);
    o.addOption([]const u8, "proj_git_lastsummary", data.__MKIT_PROJ_GIT_LASTSUMMARY__);
    o.addOption([]const u8, "proj_license", data.__MKIT_PROJ_LICENSE__);
    o.addOption([]const u8, "proj_maintainer", data.__MKIT_PROJ_MAINTAINER__);
    o.addOption([]const u8, "proj_name", data.__MKIT_PROJ_NAME__);
    o.addOption([]const u8, "proj_pkgname", data.__MKIT_PROJ_PKGNAME__);
    o.addOption([]const u8, "proj_tagline", data.__MKIT_PROJ_TAGLINE__);
    o.addOption([]const u8, "proj_vcs_browser", data.__MKIT_PROJ_VCS_BROWSER__);
    o.addOption([]const u8, "proj_version", data.__MKIT_PROJ_VERSION__);
    m.addOptions("mkit_meta", o);
}
